/*
 * Copyright (c) 2011 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Thu Apr  7 2011
 */
#ifndef H_DC1394_COMMON
#define H_DC1394_COMMON

#include "viam/viam_acq_driver.h"

#include <dc1394/control.h>

typedef struct viamgen1394_camera_t {
  dc1394camera_t *handle;		/**< libdc1394 camera handle. */
  dc1394featureset_t featset;		/**< camera features. */
  dc1394video_frame_t *frame;		/**< last captured frame. */
  uint32_t xmit;			/**< transmission requests */
} *viamgen1394_camera_t;

int	viamgen1394_init(viam_handle_t vh);
int	viamgen1394_fini(viam_handle_t vh);

int	viamgen1394_hardware_list(viam_handle_t vh,
                void (*print)(dc1394camera_t *));
viamgen1394_camera_t
	viamgen1394_hardware_attach(viam_handle_t vh, const char *name,
                uint64_t uid);
void	viamgen1394_hardware_detach(viam_handle_t vh, viamgen1394_camera_t hw);

int	viamgen1394_camera_configure(viam_handle_t vh, viamgen1394_camera_t hw,
                viam_hwcrop_t crop, uint32_t w, uint32_t h,
                viam_hwformat_t fmt, double fps, viam_hwtrigger_t trigger);
int	viamgen1394_camera_unconfigure(viam_handle_t vh,
                viamgen1394_camera_t hw);

int	viamgen1394_datatransmit(viam_handle_t vh, viamgen1394_camera_t hw,
                viam_onoff_t on);
int	viamgen1394_dataflush(viam_handle_t vh, viamgen1394_camera_t hw);

int	viamgen1394_capture(viam_handle_t vh, viamgen1394_camera_t hw);
char *	viamgen1394_capturedata(viam_handle_t vh, viamgen1394_camera_t hw,
		size_t *bytes, size_t *rowstride);
struct timeval
	viamgen1394_capturetimestamp(viam_handle_t vh, viamgen1394_camera_t hw);
int	viamgen1394_capturedone(viam_handle_t vh, viamgen1394_camera_t hw);

int	viamgen1394_queryhwfeature(viam_handle_t vh, viamgen1394_camera_t hw,
                viam_filtertype_t filter, unsigned int *methods,
                unsigned int *aumodes);
int	viamgen1394_sethwfeaturemode(viam_handle_t vh, viamgen1394_camera_t hw,
                viam_filtertype_t filter, viam_filtermethod_t method,
                viam_filterautomode_t aumode);
int	viamgen1394_sethwfeaturevalue(viam_handle_t vh, viamgen1394_camera_t hw,
                viam_filtertype_t filter, const viam_filterargs_obj args);

int	viamgen1394_fps(viam_hwfps_t swfps, double *hwfps);

extern dc1394_t *dc1394h;		/**< general libdc134 handle */

#endif /* H_DC1394_COMMON */
