/* $LAAS: config.c 2011/02/08 14:40:31 mallet $ */

/* 
 * Copyright (c) 2007-2008,2011 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Tue Jan 23 2007
 */

#include <stdlib.h>
#include <err.h>

#include "viam_private.h"

#ifdef VIAM_OPENCV_SUPPORT
# ifndef CV_INLINE
#  define CV_INLINE	static inline
# endif
# define CV_NO_BACKWARD_COMPATIBILITY
# include <cxtypes.h>
# include <cv.h>
#endif


/* --- local data ---------------------------------------------------- */

struct viam_handle {
  viam_camera_t cameras;	/**< list of cameras */
  viam_image_ref images;		/**< list of images */
  viam_bank_t banks;		/**< list of banks */
  viam_filter_t filters;	/**< list of filters */

  int error;			/**< last error code */
};


/* --- viam_init ----------------------------------------------------- */

/** \brief Initialize viam library
 *
 * \return library handle
 */

viam_handle_t
viam_init()
{
  viam_handle_t vh;

#ifdef VIAM_OPENCV_SUPPORT
  /* configure opencv error handlers */
  cvSetErrMode(CV_ErrModeParent);
  cvRedirectError(cvStdErrReport, NULL, NULL);
#endif

  /* allocate handle */
  vh = malloc(sizeof(*vh));
  if (!vh) return NULL;

  vh->cameras = NULL;
  vh->images = NULL;
  vh->banks = NULL;
  vh->filters = NULL;

  return vh;
}


/* --- viam_release -------------------------------------------------- */

/** \brief Shutdown cameras
 *
 * Destroy configuration and close hardware
 *
 * \param[in] vh	Library handle.
 */

void
viam_release(viam_handle_t vh)
{
  while(viam_config_banks(vh))
    viam_bank_delete(vh, viam_config_banks(vh));
  while(viam_config_cameras(vh))
    viam_camera_delete(vh, viam_config_cameras(vh));

  free(vh);
}


/* --- viam_config mutators and acessors ----------------------------- */

viam_camera_t
viam_config_cameras(viam_handle_t vh)
{
  return vh->cameras;
}

void
viam_config_setcameras(viam_handle_t vh, viam_camera_t camera)
{
  vh->cameras = camera;
}

viam_bank_t
viam_config_banks(viam_handle_t vh)
{
  return vh->banks;
}

void
viam_config_setbanks(viam_handle_t vh, viam_bank_t bank)
{
  vh->banks = bank;
}

viam_image_ref
viam_config_images(viam_handle_t vh)
{
  return vh->images;
}

void
viam_config_setimages(viam_handle_t vh, viam_image_ref image)
{
  vh->images = image;
}

viam_filter_t
viam_config_filters(viam_handle_t vh)
{
  return vh->filters;
}

void
viam_config_setfilters(viam_handle_t vh, viam_filter_t filter)
{
  vh->filters = filter;
}

viam_errorcode_t
viam_errno(viam_handle_t vh)
{
  return vh->error;
}

int
viam_seterrno(viam_handle_t vh, int eno)
{
  return vh->error = eno;
}
